// Copyright 2019 Intel Corporation
// SPDX-License-Identifier: Apache-2.0

#pragma once

#define OPENVKL_VERSION_MAJOR 2
#define OPENVKL_VERSION_MINOR 0
#define OPENVKL_VERSION_PATCH 1
#define OPENVKL_VERSION "2.0.1"
